﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 커스텀 탭 컨트롤
    /// </summary>
    public partial class CustomTabControl : TabControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 현재 컨텐트 프리젠터
        /// </summary>
        private ContentPresenter currentContentPresenter = null;

        /// <summary>
        /// 이전 컨텐트 프리젠터
        /// </summary>
        private ContentPresenter beforeContentPresenter = null;

        /// <summary>
        /// FADE IN 애니메이션
        /// </summary>
        private DoubleAnimation fadeInAnimation;

        /// <summary>
        /// FADE OUT 애니메이션
        /// </summary>
        private DoubleAnimation fadeOutAnimation;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomTabControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomTabControl()
        {
            InitializeComponent();

            this.fadeInAnimation = new DoubleAnimation(0, 1, Duration.Automatic, FillBehavior.HoldEnd)
            {
                SpeedRatio        = 2,
                AccelerationRatio = 0.7
            };

            this.fadeOutAnimation = new DoubleAnimation(1, 0, Duration.Automatic, FillBehavior.HoldEnd)
            {
                SpeedRatio        = 2,
                AccelerationRatio = 0.7
            };

            Loaded                          += TabControl_Loaded;
            SelectionChanged                += TabControl_SelectionChanged;
            this.fadeOutAnimation.Completed += fadeOutAnimation_Completed;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 탭 컨트롤 로드시 처리하기 - TabControl_Loaded(sender, e)

        /// <summary>
        /// 탭 컨트롤 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void TabControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.currentContentPresenter = GetTemplateChild("currentContentPresenter") as ContentPresenter;
            this.beforeContentPresenter  = GetTemplateChild("beforeContentPresenter" ) as ContentPresenter;

            this.currentContentPresenter.Content = SelectedContent;
        }

        #endregion
        #region 탭 컨트롤 선택 변경시 처리하기 - TabControl_SelectionChanged(sender, e)

        /// <summary>
        /// 탭 컨트롤 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void TabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(currentContentPresenter == null)
            {
                return;
            }

            this.currentContentPresenter.Content = (e.AddedItems[0]   as ContentControl).Content;
            this.beforeContentPresenter.Content  = (e.RemovedItems[0] as ContentControl).Content;

            this.beforeContentPresenter.Visibility = Visibility.Visible;

            this.currentContentPresenter.BeginAnimation(ContentPresenter.OpacityProperty, fadeInAnimation );
            this.beforeContentPresenter.BeginAnimation (ContentPresenter.OpacityProperty, fadeOutAnimation);
        }

        #endregion
        #region FADE OUT 애니메이션 완료시 처리하기 - fadeOutAnimation_Completed(sender, e)

        /// <summary>
        /// FADE OUT 애니메이션 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fadeOutAnimation_Completed(object sender, EventArgs e)
        {
            this.beforeContentPresenter.Visibility = Visibility.Collapsed;
        }

        #endregion
    }
}